@@ -744,6 +744,24 @@ static int register_casts()
         MARK_UNUSED(data);                                              \
     }
 
+#define ULOOP_MODF(func_name, inner_func, type_out, type_a, type_b)     \
+    static void func_name(char **args, const npy_intp *dimensions,      \
+                          const npy_intp* steps, void *data)            \
+    {                                                                   \
+        const npy_intp n = dimensions[0];                               \
+        const npy_intp as = steps[0] / sizeof(type_a),                  \
+                       bs = steps[1] / sizeof(type_b),                  \
+                       os = steps[2] / sizeof(type_out);                \
+        const type_a *a = (const type_a *)args[0];                      \
+        type_b *b = (type_b *)args[2];                                  \
+        type_out *out = (type_out *)args[1];                            \
+                                                                        \
+        for (npy_intp i = 0; i < n; ++i) {                              \
+            out[i * os] = inner_func(a[i * as], &b[i * bs]);            \
+        }                                                               \
+        MARK_UNUSED(data);                                              \
+    }
+
 ULOOP_BINARY(u_addqd, addqd, ddouble, ddouble, double)
 ULOOP_BINARY(u_subqd, subqd, ddouble, ddouble, double)
 ULOOP_BINARY(u_mulqd, mulqd, ddouble, ddouble, double)
@@ -783,10 +801,17 @@ ULOOP_BINARY(u_fminqd, fminqd, ddouble, ddouble, double)
 ULOOP_BINARY(u_fmaxqd, fmaxqd, ddouble, ddouble, double)
 ULOOP_BINARY(u_fmindq, fmindq, ddouble, double, ddouble)
 ULOOP_BINARY(u_fmaxdq, fmaxdq, ddouble, double, ddouble)
+ULOOP_BINARY(u_atan2qd, atan2qd, ddouble, ddouble, double)
+ULOOP_BINARY(u_atan2dq, atan2dq, ddouble, double, ddouble)
+ULOOP_BINARY(u_atan2qq, atan2qq, ddouble, ddouble, ddouble)
+ULOOP_BINARY(u_powqd, powqd, ddouble, ddouble, double)
+ULOOP_BINARY(u_powdq, powdq, ddouble, double, ddouble)
+ULOOP_BINARY(u_powqq, powqq, ddouble, ddouble, ddouble)
 ULOOP_BINARY(u_hypotqq, hypotqq, ddouble, ddouble, ddouble)
 ULOOP_BINARY(u_hypotdq, hypotdq, ddouble, double, ddouble)
 ULOOP_BINARY(u_hypotqd, hypotqd, ddouble, ddouble, double)
-
+ULOOP_BINARY(u_ldexpqi, ldexpqi, ddouble, ddouble, int)
+ULOOP_MODF(u_modfqq, modfqq, ddouble, ddouble, ddouble)
 ULOOP_UNARY(u_signbitq, signbitq, bool, ddouble)
 ULOOP_UNARY(u_signq, signq, ddouble, ddouble)
 ULOOP_UNARY(u_isfiniteq, isfiniteq, bool, ddouble)
@@ -806,6 +831,13 @@ ULOOP_UNARY(u_expm1q, expm1q, ddouble, ddouble)
 ULOOP_UNARY(u_logq, logq, ddouble, ddouble)
 ULOOP_UNARY(u_sinq, sinq, ddouble, ddouble)
 ULOOP_UNARY(u_cosq, cosq, ddouble, ddouble)
+ULOOP_UNARY(u_tanq, tanq, ddouble, ddouble)
+ULOOP_UNARY(u_atanq, atanq, ddouble, ddouble)
+ULOOP_UNARY(u_acosq, acosq, ddouble, ddouble)
+ULOOP_UNARY(u_asinq, asinq, ddouble, ddouble)
+ULOOP_UNARY(u_atanhq, atanhq, ddouble, ddouble)
+ULOOP_UNARY(u_acoshq, acoshq, ddouble, ddouble)
+ULOOP_UNARY(u_asinhq, asinhq, ddouble, ddouble)
 ULOOP_UNARY(u_sinhq, sinhq, ddouble, ddouble)
 ULOOP_UNARY(u_coshq, coshq, ddouble, ddouble)
 ULOOP_UNARY(u_tanhq, tanhq, ddouble, ddouble)
@@ -872,6 +904,55 @@ error:
     return false;
 }
 
+static int register_ldexp(PyUFuncGenericFunction func, int ret_dtype,
+                          const char *name)
+{
+    PyUFuncObject *ufunc;
+    int *arg_types = NULL, retcode = 0;
+
+    ufunc = (PyUFuncObject *)PyObject_GetAttrString(numpy_module, name);
+    if (ufunc == NULL) goto error;
+
+    arg_types = PyMem_New(int, 3);
+    if (arg_types == NULL) goto error;
+
+    arg_types[0] = type_num;
+    arg_types[1] = NPY_INTP;
+    arg_types[2] = ret_dtype;
+    retcode = PyUFunc_RegisterLoopForType(ufunc, type_num,
+                                          func, arg_types, NULL);
+    if (retcode < 0) goto error;
+    return true;
+
+error:
+    return false;
+}
+
+static int register_modf(PyUFuncGenericFunction func, int ret_dtype,
+                          const char *name)
+{
+    PyUFuncObject *ufunc;
+    int *arg_types = NULL, retcode = 0;
+
+    ufunc = (PyUFuncObject *)PyObject_GetAttrString(numpy_module, name);
+    if (ufunc == NULL) goto error;
+
+    arg_types = PyMem_New(int, 4);
+    if (arg_types == NULL) goto error;
+
+    arg_types[0] = type_num;
+    arg_types[1] = type_num;
+    arg_types[2] = ret_dtype;
+    arg_types[3] = ret_dtype;
+    retcode = PyUFunc_RegisterLoopForType(ufunc, type_num,
+                                          func, arg_types, NULL);
+    if (retcode < 0) goto error;
+    return true;
+
+error:
+    return false;
+}
+
 static int register_ufuncs()
 {
     bool ok = register_unary(u_negq, type_num, "negative")
@@ -892,14 +973,24 @@ static int register_ufuncs()
         && register_unary(u_logq, type_num, "log")
         && register_unary(u_sinq, type_num, "sin")
         && register_unary(u_cosq, type_num, "cos")
+        && register_unary(u_tanq, type_num, "tan")
+        && register_unary(u_atanq, type_num, "arctan")
+        && register_unary(u_acosq, type_num, "arccos")
+        && register_unary(u_asinq, type_num, "arcsin")
+        && register_unary(u_atanhq, type_num, "arctanh")
+        && register_unary(u_acoshq, type_num, "arccosh")
+        && register_unary(u_asinhq, type_num, "arcsinh")
         && register_unary(u_sinhq, type_num, "sinh")
         && register_unary(u_coshq, type_num, "cosh")
         && register_unary(u_tanhq, type_num, "tanh")
         && register_unary(u_signq, type_num, "sign")
+        && register_ldexp(u_ldexpqi, type_num, "ldexp")
+        && register_modf(u_modfqq, type_num, "modf")
         && register_binary(u_adddq, u_addqd, u_addqq, type_num, "add")
         && register_binary(u_subdq, u_subqd, u_subqq, type_num, "subtract")
         && register_binary(u_muldq, u_mulqd, u_mulqq, type_num, "multiply")
         && register_binary(u_divdq, u_divqd, u_divqq, type_num, "true_divide")
+        && register_binary(u_powdq, u_powqd, u_powqq, type_num, "power")
         && register_binary(u_equaldq, u_equalqd, u_equalqq, NPY_BOOL, "equal")
         && register_binary(u_notequaldq, u_notequalqd, u_notequalqq, NPY_BOOL,
                            "not_equal")
@@ -913,6 +1004,7 @@ static int register_ufuncs()
         && register_binary(u_fmaxdq, u_fmaxqd, u_fmaxqq, type_num, "fmax")
         && register_binary(u_fmindq, u_fminqd, u_fminqq, type_num, "minimum")
         && register_binary(u_fmaxdq, u_fmaxqd, u_fmaxqq, type_num, "maximum")
+        && register_binary(u_atan2dq, u_atan2qd, u_atan2qq, type_num, "arctan2")
         && register_binary(u_copysigndq, u_copysignqd, u_copysignqq, type_num,
                            "copysign")
         && register_binary(u_hypotdq, u_hypotqd, u_hypotqq, type_num, "hypot");
